@@ -2,6 +2,10 @@
 #'
 #' * `expect_setequal(x, y)` tests that every element of `x` occurs in `y`,
 #'    and that every element of `y` occurs in `x`.
+#' * `expect_contains(x, y)` tests that `x` contains every element of `y`
+#'   (i.e. `y` is a subset of `x`).
+#' * `expect_in(x, y)` tests every element of `x` is in `y`
+#'   (i.e. `x` is a subset of `y`).
 #' * `expect_mapequal(x, y)` tests that `x` and `y` have the same names, and
 #'    that `x[names(y)]` equals `y`.
 #'
@@ -117,3 +121,53 @@ check_names_ok <- function(x, label) {
     stop("All elements in `", label, "` must be named")
   }
 }
+
+#' @export
+#' @rdname expect_setequal
+expect_contains <- function(object, expected) {
+  act <- quasi_label(enquo(object), arg = "object")
+  exp <- quasi_label(enquo(expected), arg = "expected")
+
+  if (!is_vector(act$val) || !is_vector(exp$val)) {
+    abort("`object` and `expected` must both be vectors")
+  }
+
+  exp_miss <- !exp$val %in% act$val
+
+  if (any(exp_miss)) {
+    fail(paste0(
+      act$lab, " (`actual`) doesn't fully contain all the values in ", exp$lab, " (`expected`).\n",
+      paste0("* Missing from `actual`: ",  values(exp$val[exp_miss]), "\n"),
+      paste0("* Present in `actual`:   ",  values(act$val), "\n")
+    ))
+  } else {
+    succeed()
+  }
+
+  invisible(act$val)
+}
+
+#' @export
+#' @rdname expect_setequal
+expect_in <- function(object, expected) {
+  act <- quasi_label(enquo(object), arg = "object")
+  exp <- quasi_label(enquo(expected), arg = "expected")
+
+  if (!is_vector(act$val) || !is_vector(exp$val)) {
+    abort("`object` and `expected` must both be vectors")
+  }
+
+  act_miss <- !act$val %in% exp$val
+
+  if (any(act_miss)) {
+    fail(paste0(
+      act$lab, " (`actual`) isn't fully contained within ", exp$lab, " (`expected`).\n",
+      paste0("* Missing from `expected`: ",  values(act$val[act_miss]), "\n"),
+      paste0("* Present in `expected`:   ",  values(exp$val), "\n")
+    ))
+  } else {
+    succeed()
+  }
+
+  invisible(act$val)
+}
