@@ -3,20 +3,12 @@
 #' @description
 #' `r lifecycle::badge("superseded")`
 #'
-#' `with_mock()` and `local_mock()` are superseded in favour of the more
-#' rigorous techniques found in the [mockr](https://krlmlr.github.io/mockr/)
-#' and [mockery](https://github.com/r-lib/mockery#mockery) packages.
+#' `with_mock()` and `local_mock()` are superseded in favour of
+#' [with_mocked_bindings()] and [local_mocked_bindings()].
 #'
-#' Mocking allows you to temporary replace the implementation of functions
-#' within a package, which useful for testing code that relies on functions
-#' that are slow, have unintended side effects or access resources that may
-#' not be available when testing.
-#'
-#' This works by using some C code to temporarily modify the mocked function
-#' _in place_. On exit, all functions are restored to their previous state.
-#' This is somewhat abusive of R's internals so use with care. In particular,
-#' functions in base packages cannot be mocked; to work aroud you'll need to
-#' make a wrapper function in your own package..
+#' These works by using some C code to temporarily modify the mocked function
+#' _in place_. This is abusive of R's internals, which makes it dangerous, and
+#' no longer recommended.
 #'
 #' @section 3rd edition:
 #' `r lifecycle::badge("deprecated")`
@@ -34,9 +26,7 @@
 #' @return The result of the last unnamed parameter
 #' @export
 with_mock <- function(..., .env = topenv()) {
-  edition_deprecate(3, "with_mock()",
-    "Please use mockr or mockery packages instead"
-  )
+  edition_deprecate(3, "with_mock()", "Please use with_mocked_bindings() instead")
 
   dots <- eval(substitute(alist(...)))
   mock_qual_names <- names(dots)
@@ -71,9 +61,7 @@ with_mock <- function(..., .env = topenv()) {
 #' @export
 #' @rdname with_mock
 local_mock <- function(..., .env = topenv(), .local_envir = parent.frame()) {
-  edition_deprecate(3, "local_mock()",
-    "Please use mockr or mockery packages instead"
-  )
+  edition_deprecate(3, "local_mock()", "Please use local_mocked_bindings() instead")
 
   mocks <- extract_mocks(list(...), .env = .env)
   on_exit <- bquote(
@@ -151,8 +139,3 @@ reset_mock <- function(mock) {
 is_base_pkg <- function(x) {
   x %in% rownames(utils::installed.packages(priority = "base"))
 }
-
-test_mock1 <- function() {
-  test_mock2()
-}
-test_mock2 <- function() 10
