@@ -1,7 +1,3 @@
-#' @include reporter.R
-NULL
-
-
 methods::setOldClass("proc_time")
 
 #' List reporter: gather all test results along with elapsed time and
@@ -22,6 +18,7 @@ ListReporter <- R6::R6Class("ListReporter",
 
     initialize = function() {
       super$initialize()
+      self$capabilities$parallel_support <- TRUE
       self$results <- Stack$new()
     },
 
@@ -31,14 +28,25 @@ ListReporter <- R6::R6Class("ListReporter",
     },
 
     add_result = function(context, test, result) {
-      if (!is.null(self$current_expectations)) {
-        self$current_expectations$push(result)
+      if (is.null(self$current_expectations)) {
+        # we received a result outside of a test:
+        # could be a bare expectation or an exception/error
+        if (!inherits(result, 'error')) {
+          return()
+        }
+        self$current_expectations <- Stack$new()
       }
+
+      self$current_expectations$push(result)
     },
 
     end_test = function(context, test) {
       elapsed <- as.double(proc.time() - self$current_start_time)
 
+      results <- list()
+      if (!is.null(self$current_expectations))
+        results <- self$current_expectations$as_list()
+
       self$results$push(list(
         file =    self$current_file %||% NA_character_,
         context = context,
@@ -46,17 +54,49 @@ ListReporter <- R6::R6Class("ListReporter",
         user =    elapsed[1],
         system =  elapsed[2],
         real =    elapsed[3],
-        results = self$current_expectations$as_list()
+        results = results
       ))
+
+      self$current_expectations <- NULL
     },
 
     start_file = function(name) {
       self$current_file <- name
     },
 
+    end_file = function() {
+      # fallback in case we have errors but no expectations
+      self$end_context(self$current_file)
+    },
+
+    end_context = function(context) {
+      results <- self$current_expectations
+      if (is.null(results)) {
+        return()
+      }
+
+      self$current_expectations <- NULL
+
+      # look for exceptions raised outside of tests
+      # they happened just before end_context since they interrupt the test_file execution
+      results <- results$as_list()
+      if (length(results) == 0) return()
+
+      self$results$push(list(
+        file =    self$current_file %||% NA_character_,
+        context = context,
+        test =    NA_character_,
+        user =    NA_real_,
+        system =  NA_real_,
+        real =    NA_real_,
+        results = results
+       ))
+    },
+
     get_results = function() {
       testthat_results(self$results$as_list())
     }
+
   )
 )
 
@@ -95,7 +135,16 @@ any_warnings <- function(res) {
 #' @export
 as.data.frame.testthat_results <- function(x, ...) {
   if (length(x) == 0) {
-    return(data.frame())
+    return(
+      data.frame(
+        file = character(0), context = character(0), test = character(0),
+        nb = integer(0), failed = integer(0), skipped = logical(0),
+        error = logical(0), warning = integer(0),
+        user = numeric(0), system = numeric(0), real = numeric(0),
+        passed = integer(0), result = list(),
+        stringsAsFactors = FALSE
+      )
+    )
   }
 
   rows <- lapply(x, summarize_one_test_results)
